using System;
using UnityEngine;

//,       ,           
//    ,    PostProcessor http://answers.unity3d.com/questions/24254/any-way-to-find-original-prefab-name-by-script.html

public class RendererNameAttribute : Attribute
{
    public RendererNameAttribute(string value)
    {
        Value = value;
    }

    public string Value { get; private set; }
}


public enum MainRendererName
{
    [RendererName("skeleton")]
    Skeleton,

    [RendererName("geoSkin")]
    Warrior
}

public static class RendererDeterminer
{
    /// <summary>
    ///  Renderer   
    /// </summary>
    public static Renderer Determine(Transform transform, MainRendererName rendererName)
    {
        string name = GetRendererName(rendererName);

        Renderer renderer = transform.FindChild(name).GetComponent<Renderer>();
        if (renderer == null)
            Debug.LogError(string.Format("renderer for {0} transform not found", transform.name));
        return renderer;
    }

    static string GetRendererName(MainRendererName renderer)
    {
        object[] attribs = typeof(MainRendererName)
            .GetField(renderer.ToString())
            .GetCustomAttributes(typeof(RendererNameAttribute), false);

        if (attribs != null && attribs.Length > 0)
            return ((RendererNameAttribute)attribs[attribs.Length - 1]).Value;

        Debug.LogError("Conversion for this renderer is not implemented");
        return null;
    }
}